;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x4dcd7f00d601ec2ff2c2fd50a0863fdf . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe2ef3d74763d1f3e53ffa500f503c216 . "./code.cg2js/cg-printing.fasl")
(#x992af50919c133295ec536ec08536f25 . "./code.cg2js/cg-caret.fasl")
(#x7c5489654cf9466e0a400deb51e73153 . "./code.cg2js/cg-item-list.fasl")
(#x6ed6fe0f73da80aa8f7213953b697395 . "./code.cg2js/cg-progress-indicator.fasl")
(#x22ff70401804cd2649a39e1984c8e295 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1cb255ca569841400bbb9c296964665f . "./code.cg2js/cg-outline.fasl")
(#x95be17366baa599f44099dec622818cd . "./code.cg2js/cg-palette.fasl")
(#x291e2a77a2cf5ae5bc201f72641b60f8 . "./code.cg2js/cg-selected-object.fasl")
(#x58a9530d61dcbfe17db41bbbe3f2f289 . "./code.cg2js/cg-status-bar.fasl")
(#xd5640e5cf9ed3f76d160103df6dec495 . "./code.cg2js/cg-wrap-string.fasl")
(#x2de8e45d3bbf2828cc9f1a05f2bb6a0c . "./code.cg2js/cg-fill-texture.fasl")
(#xa2afa3cd25982feb68a5beda2f07a2bc . "./code.cg2js/cg-layout.fasl")
(#xf21fc2779d5348441a4663cfb0ca3a6b . "./code.cg2js/cg-timer.fasl")
(#xa0062c304a248d095dc57b33b1bcf4d9 . "./code.cg2js/cg-dialog-item.fasl")
(#x98de3e59a9f49da00e84306d117d681c . "./code.cg2js/cg-button.fasl")
(#x5a6329c641a7c3551e1a67f09c02868e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x930930ad2018148d4ec14b5d281d1425 . "./code.cg2js/cg-tray.fasl")
(#x40f89be8a4d443f177c14c5e698a8f0c . "./code.cg2js/cg-plot-widget.fasl")
(#x53a73e7b7cf31b2d39d0dc5c083851d8 . "./code.cg2js/cg-pixmap.fasl")
(#x2c43d23726a324a5e4e0e5dd394ec82f . "./code.cg2js/cg-font-dialog.fasl")
(#x039d3b8fceee92b7623ae994a4337967 . "./code.cg2js/cg-os-widget.fasl")
(#x38abb88fd99544ec6e6671148292fc1a . "./code.cg2js/cg-text-or-combo.fasl")
(#x50b7cdc720710435c578db9d19c09bf2 . "./code.cg2js/cg-trackbar.fasl")
(#xd3d627c00d3eb77edf08dac154e46f27 . "./code.cg2js/cg-up-down-control.fasl")
(#x3f368c784ad1f648738209f6f85a7500 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf4a911397cdd5b40f33309c643107589 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xba08920748984f682d07a75078626888 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x7ce7f94f71a6d7dee03e850714177799 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x70b92f5ac0f819ad85145e29077e4527 . "./code.cg2js/cg-chart-widget.fasl")
(#x7e1313328b5f94fde03dcb921e4e99ea . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xbd2131a24e162d53085aa34bb958ce3f . "./code.cg2js/cg-group-box.fasl")
(#x091f396659b0a61fd8d99ec93e6de767 . "./code.cg2js/cg-static-text.fasl")
(#x2a28f38a09283ce56b7e8b62f71cd18a . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa74958a50b1bc5c40ab0d1cb48dc3182 . "./code.cg2js/cg-color-dialog.fasl")
(#xf62f04d443397f701475bf85e6e1f4a1 . "./code.cg2js/cg-intersect.fasl")
(#x9fe870bed7a77a490af4f06f72a4c8f8 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x38b34e83c5414139141873ffb1bce8d3 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x00da5d03e714baccf88b66ef758f6208 . "./code.cg2js/cg-html-dialog.fasl")
(#xa7d20039b6a08f7d714e9fd5c7314453 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa6425f99ea0ce4efe185dc6451b5d3ea . "./code.cg2js/cg-class-grid.fasl")
(#x499031bce9842fcfa7abc10fd5ee571d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x97cfde882128034feb1619817bc5770b . "./code.cg2js/cg-calendar.fasl")
(#x74de6fb24b8dde237a8bf567bb8e3a35 . "./code.cg2js/cg-editable-text.fasl")
(#x00dc266685beeff8069573c862804193 . "./code.cg2js/cg-property.fasl")
(#x6e6ce73fbe76476be6efd2b53120a282 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x8ce7678e2d2455a69e07e52f16349b23 . "./code.cg2js/cg-grid-widget.fasl")
(#x777dfba7e1167d22065982d754d82f18 . "./code.cg2js/cg-paren-matching.fasl")
(#x61408552b0bb839903a339b4a4cc7836 . "./code.cg2js/cg-dev.fasl")
(#xd6fec3e08611071f6a319b124806a217 . "./code.cg2js/cg-drawable.fasl")
(#x8ceed48b21e7568f60f08202feda3e8e . "./code.cg2js/cg-curve.fasl")
(#xa46215f422a74f8c890029dfaa9e4bd2 . "./code.cg2js/cg-lisp-widget.fasl")
(#xef0b05e32740f4adb23bb53f3408cfcf . "./code.cg2js/cg-dde-utils.fasl")
(#xf2e69691953da015ad87312e91e37159 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x61e3d6b117557eed9b64d558aa0e1f8e . "./code.cg2js/cg-object-editor-layout.fasl")
(#xd4993b041467894d3cd347044c1f2650 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x156b575c0eb6812d1e81b32053b6ffa4 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x98341d159d42117c94bab7fef4934de6 . "./code.cg2js/cg-lamp.fasl")
(#x0f7d5d58d4b6524121364bd340f47ffb . "./code.cg2js/cg-common-control.fasl")
(#x36c25e96f8a06d6c50e603b61bfdef3a . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x49e72b6cd6904621ef2d8367da08502d . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1cc05a983c7095262a0b9a5870a79f90 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x8a820596be680a17e5f0e467fff8fe4f . "./code.cg2js/cg-project-window.fasl")
(#xde836bb251957be4ad36efee3761fc1b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x9d5ad9032d4857387e2a2f043b9fbdcb . "./code.cg2js/cg-class-support.fasl")
(#xe99689a86bccc201406ee60414656bb8 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0e266ad5bbd94f4ebb4f8fd961392a57 . "./code.cg2js/cg-yes-no-list.fasl")
(#x1bc09f6cc2d286e66c9f9a10813d1429 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x12aaea973f0418b8a3cd456cabf497b3 . "./code.cg2js/cg-string-dialog.fasl")
(#x29a107e35a77167b9339d2806333639a . "./code.cg2js/cg-menu-tooltip.fasl")
(#x7fba87d03a7a6ab21b5de83dd122458e . "./code.cg2js/cg-dropping-outline.fasl")
(#xa844003facfdeab608d33f3e714105aa . "./code.cg2js/cg.fasl")
(#x628b0e230edad739ad98d7243bdfa900 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xef18d36f4bc3ceb17d767545910f592b . "./code.cg2js/cg-scaling-stream.fasl")
(#x4f16a0387c20120d080b5b568c3b0c5a . "./code.cg2js/cg-yes-no-string.fasl")
(#xbab6eac5e1bb61fac17c1e2724d23005 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9178f2b33ad54021b9aebc1a9ff8d0e4 . "./code.cg2js/cg-web-browser.fasl")
(#x0a5636789cccf74d5418c6b5cd3c328f . "./code.cg2js/cg-clipboard-stack.fasl")
(#x7f97942b89b511eaf531b60c5e196873 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xc283254afb36692e92801a83382134a9 . "./code.cg2js/cg-message-dialog.fasl")
(#x4883ece4fc918b95741e44dc94bec054 . "./code.cg2js/cg-comtab.fasl")
(#x734eb99946111c67755c0530fb9f0297 . "./code.cg2js/cg-base.fasl")
(#xf1c2e4aab2756c259b3da9ac929b8f26 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbc6d33bc5d77d491e6e77c07a6a60b1f . "./code.cg2js/cg-text-edit-pane.fasl")
(#xaac60c9d4fa035b7d3d0695912f49d61 . "./code.cg2js/cg-utility-dialog.fasl")
(#x991d66c4219bb54a118c6be41d5cf2cd . "./code.cg2js/cg-object-editor.fasl")
(#x0e6bd6fb671b560cc499851a2d2d0689 . "./code.cg2js/cg-edit-in-place.fasl")
(#xa534eb7315f6ba49c887843d0a0a9202 . "./code.cg2js/cg-radio-button.fasl")
(#x4558457501e1febb200e936253488aaf . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x13479e5ec2fd2387fc13c9f782c761d9 . "./code.cg2js/cg-nodes.fasl")
(#xf00914cf9043c09975a97c626557419e . "./code.cg2js/cg-get-position.fasl")
(#x17c39f400e3fbf77f6f51905b8a1cd65 . "./code.cg2js/cg-directory-dialog.fasl")
(#xe05cc5dd96471fe5ade945e196245c86 . "./code.cg2js/cg-lisp-text.fasl")
(#xd78ef8e49071641957b5bdb58ec01615 . "./code.cg2js/cg-combo-box.fasl")
(#xfc0fcc554c50ae3c0ba7e7299b0fdb83 . "./code.cg2js/cg-text-widget.fasl")
(#xfac24da14671c195c006e7b575a46a5b . "./code.cg2js/cg-list-view.fasl")
(#xd0d79e3a62ee534dc174fd7eb37f6890 . "./code.cg2js/cg-graphics-context.fasl")
(#x3354c1419588e0ae07adf201722d4785 . "./code.cg2js/cg-get-pixmap.fasl")
(#x591c63941e00169d8271d4d3fa3e7999 . "./code.cg2js/cg-menu.fasl")
(#x4d19ede8c25240d25a73c696e0a54858 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xdf5506ac0aa512d60486df7186f36c49 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xe7c068a4b5fd5fe4a180e99c32799590 . "./code.cg2js/cg-tooltip.fasl")
(#x0605e475b05c5e1270ca6300bce531e5 . "./code.cg2js/cg-html-widget.fasl")
(#x23030bd56fadc9d77551bd9bd516b2a0 . "./code.cg2js/cg-choice-list.fasl")
(#xda3fe688ca367cf0f249b202e9b5a14c . "./code.cg2js/cg-icon.fasl")
(#x6ec6662e6caf35c6dbacb590fba257fb . "./code.cg2js/cg-acache.fasl")
(#xfd71f5a80b0cbf47dca79aba5d4463e4 . "./code.cg2js/cg-template-string.fasl")
(#x240df47d111264def1b61b8ec40ec453 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1a089b1efe76dba18eb12e2ea2ebb6d8 . "./code.cg2js/cg-toggling-widget.fasl")
(#xbe2897c9227be2a63c5c2076bd4af819 . "./code.cg2js/cg-tab-control.fasl")
(#x6df62abd8c82c77f361fdae588fb268b . "./code.cg2js/cg-split-bar.fasl")
(#x4ec9c10d5a2962721b9473ed0f3075a1 . "./code.cg2js/cg-picture-widget.fasl")
(#xc7d7663f4704f911b384a5f821d73fa4 . "./code.cg2js/cg-player.fasl")
(#x7b7c099a1b50285eb6e1adc928939bb3 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8ccfdb8a8337bc81e209c6c3ba8070ff . "./code.cg2js/cg-choose-printer.fasl")
(#x15c03bfb187f2ed410c1fa8974300a35 . "./code.cg2js/cg-os-window.fasl")
(#xe87f0d65c3fe7ba3e67ebd75ca7d7637 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf2fd307d64ceef1516ccde0e1554d0eb . "./code.cg2js/cg-clipboard.fasl")
(#x50467791d118958e09d6fc1cc5627304 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8775056f9bd45799882c755259926b32 . "./code.cg2js/cg-hotspot.fasl")
(#xf9a2438b153665ebe2ec701eba71d2db . "./code.cg2js/cg-check-box.fasl")
(#xfe32e06d839bff8b445393a8b0f1aaa8 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x7ed6182587244ebe2d90cae460dfb9ec . "./code.cg2js/cg-file-dialog.fasl")
(#x8801f4d765bafc2bcb60cd6aa8ee8ee9 . "./code.cg2js/cg-toolbar.fasl")
(#xbae0efb52072eab8cd9fc216d4c2f444 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4dcf8f757358452a50ba42442b68f917 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x413835b8b518f41d7d81a7ed2fe7e24a . "./code.cg2js/cg-shortcut-menu.fasl")
(#x1be2fb3b0255db1b113b4111402c8f01 . "./code.idejs/ide-project.fasl")
(#x119c9f93f995dd5359a22c6fc2bf9d8a . "./code.idejs/ide-intro.fasl")
(#xbafe49a00ef94b643a64614b20247948 . "./code.idejs/ide-find-in-files.fasl")
(#x051a72710affe91ed27aa0453f1afb55 . "./code.idejs/ide-grapher.fasl")
(#x66ff653e4135bfab733c727857b776aa . "./code.idejs/ide-inspector.fasl")
(#x22b2886bcb721a6ab0986e7223c1a20a . "./code.idejs/ide-editor.fasl")
(#xa021c2483e74dcf4994e15173cc8f451 . "./code.idejs/ide-window-list.fasl")
(#x36eab4a4fbbdde0ab958c3ce09fc1271 . "./code.idejs/ide-process-browser.fasl")
(#xe7d73ae4b60a5f2e4109130d58f0dadc . "./code.idejs/ide-patch-dialog.fasl")
(#x2d68a6d78d42c3224d8c53c7ccef3315 . "./code.idejs/ide-class-grapher.fasl")
(#xce49352826969b23aa5eda57f4dfba4d . "./code.idejs/ide-base.fasl")
(#xacde456d1377dadb4cb137a5e5841b36 . "./code.idejs/ide.fasl")
(#xb25b77ba571d547144d60b0bf742c2e3 . "./code.idejs/ide-trace.fasl")
(#xed7562a11ed6467e122a90fa53200402 . "./code.idejs/ide-form.fasl")
(#x974ea5d2a737ade1ef98a2e47f4a7c40 . "./code.idejs/ide-find-definitions.fasl")
(#xca2e76470895757610bad38e16a71009 . "./code.idejs/ide-apropos.fasl")
(#x81aef3d28c5b746cff2d4ebf995d68a1 . "./code.idejs/ide-class-browser.fasl")
(#xefd8b868ab578898d92f50477cdfdbc3 . "./code.idejs/ide-navigator.fasl")
(#x90485dbe911550a41893a0b363fb5645 . "./code.idejs/ide-clipboard.fasl")
(#x16828db14c89d639611661cb612fa1ae . "./code.idejs/ide-acache.fasl")
(#x72689fe7f78bed7e6f7663e17cb84b84 . "./code.idejs/ide-keyshort.fasl")
(#x661115727f78d9c768b5ce16ce66f232 . "./code.idejs/ide-package-list.fasl")
(#x8c99e8e8b25387c57447c210ff1dcab9 . "./code.idejs/ide-dev.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
